@@ -21,23 +21,24 @@
 # THE SOFTWARE.
 
 
-from pyaes.aes import AESBlockModeOfOperation, AESSegmentModeOfOperation, AESStreamModeOfOperation
-from pyaes.util import append_PKCS7_padding, strip_PKCS7_padding
+from .aes import AESBlockModeOfOperation, AESSegmentModeOfOperation, AESStreamModeOfOperation
+from .util import append_PKCS7_padding, strip_PKCS7_padding, to_bufferable
 
-# First we add three functions to each of the modes of operations
+
+# First we inject three functions to each of the modes of operations
 #
-#    _block_an_consume(size)
-#       - Given a size determine how many bytes could be consumed in
+#    _can_consume(size)
+#       - Given a size, determine how many bytes could be consumed in
 #         a single call to either the decrypt or encrypt method
 #
-#    _block_final_encrypt(data)
+#    _final_encrypt(data)
 #       - call and return encrypt on this (last) chunk of data,
 #         padding as necessary; this will always be at least 16
 #         bytes unless the total incoming input was less than 16
 #         bytes
 #
-#    _block_final_decrypt(data)
-#       - same as _blcok_final_encrypt except for decrypt, for
+#    _final_decrypt(data)
+#       - same as _final_encrypt except for decrypt, for
 #         stripping off padding
 #
 
@@ -48,6 +49,7 @@ def _block_can_consume(self, size):
     if size >= 16: return 16
     return 0
 
+# After padding, we may have more than one block
 def _block_final_encrypt(self, data):
     data = append_PKCS7_padding(data)
     if len(data) == 32:
@@ -70,12 +72,14 @@ def _segment_can_consume(self, size):
 
 # CFB can handle a non-segment-sized block at the end using the remaining cipherblock
 def _segment_final_encrypt(self, data):
-    padded = data + (chr(0) * (self.segment_bytes - (len(data) % self.segment_bytes)))
+    faux_padding = (chr(0) * (self.segment_bytes - (len(data) % self.segment_bytes)))
+    padded = data + to_bufferable(faux_padding)
     return self.encrypt(padded)[:len(data)]
 
 # CFB can handle a non-segment-sized block at the end using the remaining cipherblock
 def _segment_final_decrypt(self, data):
-    padded = data + (chr(0) * (self.segment_bytes - (len(data) % self.segment_bytes)))
+    faux_padding = (chr(0) * (self.segment_bytes - (len(data) % self.segment_bytes)))
+    padded = data + to_bufferable(faux_padding)
     return self.decrypt(padded)[:len(data)]
 
 AESSegmentModeOfOperation._can_consume = _segment_can_consume
@@ -110,7 +114,7 @@ class BlockFeeder(object):
         self._mode = mode
         self._feed = feed
         self._final = final
-        self._buffer = ""
+        self._buffer = to_bufferable("")
 
     def feed(self, data = None):
         '''Provide bytes to encrypt (or decrypt), returning any bytes
@@ -123,14 +127,16 @@ class BlockFeeder(object):
         if self._buffer is None:
             raise ValueError('already finished feeder')
 
+        # Finalize; process the spare bytes we were keeping
         if not data:
             result = self._final(self._buffer)
             self._buffer = None
             return result
 
-        self._buffer += data
+        self._buffer += to_bufferable(data)
 
-        result = ''
+        # We keep 16 bytes around so we can determine padding
+        result = to_bufferable('')
         while len(self._buffer) > 16:
             can_consume = self._mode._can_consume(len(self._buffer) - 16)
             if can_consume == 0: break
@@ -141,11 +147,15 @@ class BlockFeeder(object):
 
 
 class Encrypter(BlockFeeder):
+    'Accepts bytes of plaintext and returns encrypted ciphertext.'
+
     def __init__(self, mode):
         BlockFeeder.__init__(self, mode, mode.encrypt, mode._final_encrypt)
 
 
 class Decrypter(BlockFeeder):
+    'Accepts bytes of ciphertext and returns decrypted plaintext.'
+
     def __init__(self, mode):
         BlockFeeder.__init__(self, mode, mode.decrypt, mode._final_decrypt)
 
